//****************************************************************
//
//
// Isabel H. Manssour e Márcio Pinho
// Disciplina: Programação 2D
// Especialização em Desenvolvimento de Jogos Digitais
//
//****************************************************************



#ifdef WIN32
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "gl/glut.h"
#endif

#ifdef __APPLE__
#include <iostream>
#include <cmath>
#include <GL/glut.h>
#endif

#include "Desenhos.h"
#include "Tools.h"
#include "Objeto.h"


// Variáveis que guardam os ângulos de rotação dos ponteiros do relógio
GLint angSec, angMin;

// Variáveis que guardam as dimensões da janela
GLfloat winXmin, winXmax;
GLfloat winYmin, winYmax;

// Variáveis logicas
bool fimJogo, imprimeCoordenadas, tentaPegar;

// Variáveis que guardam os valores de translação e a direção do helicóptero manipulado pelo usuário
GLfloat txHelicoptero, tyHelicoptero;

// Variável que guarda o ângulo de rotação das hélices
GLfloat angHelice;

// Variável que guarda o quanto o quadrado será deslocado cada vez que o usuário pressiona uma tecla de seta
GLfloat delta;

int QTD_OBJETOS;
int ObjetoAtivo;


Objeto Cenario[10];

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Função responsável por inicializar parâmetros e variáveis.
//
void Inicializa (void) {
	// Especifica a cor de fundo da janela
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);

	winXmin = -1600;
	winXmax =  1600;
	winYmin = -900;
	winYmax =  900;

	angHelice = 0;
	delta = 30;


	QTD_OBJETOS = 3;
	ObjetoAtivo = 0;

	for(int i=1; i<QTD_OBJETOS; i++) {
		Cenario[i].Init();
		Cenario[i].Translate(i*600,i*300,0);
		Cenario[i].setTipo(2); // carro
		Cenario[i].id = i;
	}
	Cenario[0].Init();
	Cenario[0].setTipo(1); // Helicoptero
	// Helicoptero.ImprimeMatriz("Apos o INIT");

	imprimeCoordenadas = false;
	tentaPegar = false;

	srand(time(0));
}


//////////////////////////////////////////////////////////////////////////////////////////////////////
// Função callback de redesenho da janela de visualização.
//
void Desenha(void) {

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// Limpa a janela
	glClear(GL_COLOR_BUFFER_BIT);

	// Desenha o chão
	glLineWidth(3);
	glColor3f(0.0f,1.0f,0.0f);
	glBegin(GL_LINES);
	glVertex2i(winXmin,winYmin);
	glVertex2i(winXmax,winYmin);
	glEnd();
	glLineWidth(1);

	char msg[100];
	if (ObjetoAtivo==0)
		strcpy(msg, "Objeto Ativo: Helicoptero");
	else sprintf(msg, "Objeto Ativo: Carro %d",ObjetoAtivo);

	DesenhaTexto(msg, winXmin+100, winYmin+100);


	// Desenha o cenario todo
	for(int i=0; i<3; i++) {
		Cenario[i].Draw();
		printf("\n");
	}
	// Imprime as coordenadas do ponto de inserção de todos os objetos do cenário
	if (imprimeCoordenadas) {
		GLfloat P1[] = {0,0,0};
		GLfloat P2[3];

		imprimeCoordenadas = false;
		printf("Posicoes do objetos:\n");
		for(int i=0; i<QTD_OBJETOS; i++) {
			printf("Objeto %d: ",i);
			Cenario[i].InstanciaPonto(P1,P2);
			for (int j=0; j<3; j++)
				printf("%7.2f ", P2[j]);
			printf("\n");
		}
		printf("======================================\n");
	}

	glutSwapBuffers();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////
// Função callback chamada quando o tamanho da janela é alterado.
//
void AlteraTamanhoJanela(GLsizei w, GLsizei h) {
	// Especifica o tamanho da viewport
	glViewport(0, 0, w, h);

	// Inicializa sistema de coordenadas
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Estabelece volume de visualização (esquerda, direita, inferior, superior)
	gluOrtho2D (winXmin, winXmax, winYmin, winYmax);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Função callback chamada pela GLUT a cada intervalo de tempo.
//
void Anima(int value) {

	glutPostRedisplay();
	glutTimerFunc(60,Anima, 1);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////
// Função callback chamada para gerenciar eventos de teclas.
//
void Teclado (unsigned char key, int x, int y) {
	// define qual dos objetos deve ser ativado (de 0 até 9)
	// O objeto 0 eh o helicptero
	if (key >= '0' && key <= '9') { // ativa objetos de 1 a 9
		ObjetoAtivo = key - '0'; // converte para o indice do objeto ativo;
		if (ObjetoAtivo >= QTD_OBJETOS)
			ObjetoAtivo = 0; // caso haj algum erro no nro do objeto, ativa o helicoptero
		printf ("---------------------------------\nObjeto Ativado: %d\n", ObjetoAtivo );
	}
	switch (key) {

	case ' ': //espaco
		imprimeCoordenadas = true;
		break;

	case 'p': // faz o helicoptero pegar um dos objetos do cenario
	case 'P':
		Cenario[0].AdicionaFilho(&Cenario[1]);
		break;
	case 'l': // faz o helicoptero largar um dos objetos do cenario
	case 'L':
		Cenario[0].RemoveFilho();
		break;
	case 27:
		exit(1);
	}
	glutPostRedisplay();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Função callback chamada para gerenciar eventos do mouse.
//
void GerenciaMouse(int button, int state, int x, int y) {

}

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Função callback chamada para gerenciar eventos de teclas especiais(F1,PgDn,...).
//
void TeclasEspeciais(int key, int x, int y) {
	if (key == GLUT_KEY_UP) { // desloca o objeto ativo para cima
		Cenario[ObjetoAtivo].Rotate(10,0,0,1);
	}
	if (key == GLUT_KEY_DOWN) { // desloca o objeto ativo para baixo
		Cenario[ObjetoAtivo].Rotate(-10,0,0,1);
	}
	if (key == GLUT_KEY_LEFT) { // desloca o objeto ativo para esquerda
		Cenario[ObjetoAtivo].Translate(-delta,0,0);
	}
	if (key == GLUT_KEY_RIGHT) { // desloca o objeto ativo para direita
		Cenario[ObjetoAtivo].Translate(delta,0,0);
	}
	glutPostRedisplay();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////
// Programa Principal.
//
int main(int argc, char *argv[]) {

	glutInit(&argc, argv);

	// Define do modo de operação da GLUT
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);


	// Especifica a posição inicial da janela GLUT
	glutInitWindowPosition(5,5);


	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(960,540);


	// Cria a janela passando como argumento o título da mesma
	glutCreateWindow("Hierarquia");


	// Registra a função callback de redesenho da janela de visualização
	glutDisplayFunc(Desenha);

	// Registra a função callback de redimensionamento da janela de visualização
	glutReshapeFunc(AlteraTamanhoJanela);

	// Registra a função callback para tratamento das teclas especiais
	glutSpecialFunc(TeclasEspeciais);

	// Registra a função callback para tratamento das teclas ASCII
	glutKeyboardFunc (Teclado);

	// Registra a função callback que gerencia os eventos do mouse
	glutMouseFunc(GerenciaMouse);

	// Registra a função callback que será chamada a cada intervalo de tempo
	glutTimerFunc(60, Anima, 1);

	// Chama a função responsável por fazer as inicializações
	Inicializa();

	// Inicia o processamento e aguarda interações do usuário
	glutMainLoop();

	return 0;
}
